from typing import Any, Dict, List, Tuple, Union

import numpy as np
import pandas as pd

from cronus_eater import _normalizer, _validator
from cronus_eater.model import TimeSeries, TimeSeriesMetadata


def find_metadata(df: pd.DataFrame, origin: str) -> List[TimeSeriesMetadata]:
    return []


def slice_dataframe(
    df: pd.DataFrame, metadata: List[TimeSeriesMetadata]
) -> List[TimeSeries]:
    return []


def find_header(df: pd.DataFrame, start_row: int, end_column: int) -> int:
    for index, value in df.iloc[start_row - 1 :: -1, end_column].items():
        if _validator.is_date_time(value):
            return int(str(index))

    return -1


def find_end_column(row: pd.Series) -> int:
    last_text_column = -1
    last_number_column = -1

    # If is a empty sequence return false
    if len(row.dropna()) == 0:
        return False

    qtd_text_sequence = 0
    # Calcule the right pattern of a time series row
    for index, value in row.items():
        if (
            _validator.is_text(value) or _validator.is_date_time(value)
        ) and last_number_column == -1:
            last_text_column = int(str(index))
            qtd_text_sequence += 1
        elif _validator.is_financial_number(value) and last_text_column != -1:
            last_number_column = int(str(index))
        elif (
            _validator.is_text(value) or _validator.is_date_time(value)
        ) and last_number_column != -1:
            break

    # if a sequence is empty means we do not have a time series pattern
    if -1 in (last_text_column, last_number_column):
        return -1

    return last_number_column


def find_start_row_index(df: pd.DataFrame, column_index: int) -> int:
    for row_index, row in df.iloc[:, column_index:].iterrows():
        if not pd.isnull(row.iloc[0]):
            if _validator.is_time_series_row(row):
                return int(str(row_index))

    return -1


def find_start_row_column(df: pd.DataFrame) -> Tuple[int, int]:
    for column_index, column in df.items():
        start_column = int(str(column_index))
        if len(column.dropna()) >= 2:
            start_row = find_start_row_index(df, int(str(column_index)))

            if start_row != -1:
                return start_row, start_column

    return -1, -1


def find_end_row_column(
    df: pd.DataFrame, start_row: int, start_column: int
) -> Tuple[int, int]:

    end_column = find_end_column(df.iloc[start_row, start_column:])
    df = df.iloc[start_row:, start_column:end_column].copy()

    end_row = -1
    for row_index, row in df.iterrows():
        if _validator.is_time_series_row(row):
            end_row = int(str(row_index))
        elif _validator.is_text_row(row):
            break

    return end_row, end_column


def clean_garbage_row(row: pd.Series) -> pd.Series:
    qtd_text = row.map(lambda value: _validator.is_text(value)).sum()
    if qtd_text >= 2:
        return row.map(lambda value: pd.NA)
    return row


def to_literal_blank(value: Any) -> Any:
    if pd.isna(value):
        return 'Vazio'
    return value


def clean_time_series_from_raw_df(
    df: pd.DataFrame, metadata: TimeSeriesMetadata
) -> pd.DataFrame:
    df.iloc[
        metadata.start_row : metadata.end_row + 1,
        metadata.start_column : metadata.end_column + 1,
    ] = np.nan
    return df.copy()


def clean_gargabe_column(
    df: pd.DataFrame, start_row: int, start_column: int
) -> pd.DataFrame:

    if start_row == -1 and start_column >= 0:
        df[df.columns[start_column]] = np.nan
        return df.copy()

    return df.copy()


def clean_gargabe_table(
    df: pd.DataFrame,
    start_row: int,
    start_column: int,
    end_row: int,
    end_column: int,
) -> pd.DataFrame:

    if (
        start_row >= 0
        and start_column >= 0
        and end_column >= 0
        and end_row >= 0
    ):
        df.iloc[start_row : end_row + 1, start_column : end_row + 1] = np.nan
        return df.copy()

    return df.copy()


def find_time_series(raw_dataframe: pd.DataFrame) -> pd.DataFrame:

    df = raw_dataframe.copy()
    dfs = []
    df_order = 1
    while True:

        # If there's no more value, finish the search
        tot_not_null = (~df.isnull()).values.sum()
        if tot_not_null == 0:
            break

        start_row, start_column = find_start_row_column(df)

        # If can find start row, clean gargabe column and starts again the search
        if start_row == -1 and start_column >= 0:
            df = clean_gargabe_column(df, start_row, start_column)
            continue
        # If there's no start column finish the search
        elif start_row == -1 and start_column == -1:
            break

        end_row, end_column = find_end_row_column(df, start_row, start_column)

        header_row = find_header(df, start_row, end_column)
        # If can't find a header row clean the table and start again the search
        if header_row == -1:
            df = clean_gargabe_table(
                df, start_row, start_column, end_row, end_column
            )
            continue
        else:
            start_row = header_row

        metadata = TimeSeriesMetadata(
            'whatever',
            'whatever',
            start_column,
            end_column,
            start_row,
            end_row,
        )
        # Copy Time Series From raw dataframe
        time_series_df = df.iloc[
            start_row : end_row + 1, start_column : end_column + 1
        ].copy()

        time_series_df = (
            time_series_df.apply(lambda row: clean_garbage_row(row), axis=1)
            .dropna(how='all', axis=0)
            .dropna(how='all', axis=1)
        )

        time_series_df[time_series_df.columns[0]] = time_series_df[
            time_series_df.columns[0]
        ].map(lambda value: to_literal_blank(value))
        time_series_df = time_series_df.applymap(
            lambda value: _normalizer.norm_blank_value(value)
        )

        time_series_df.iloc[0, 0] = 'Index'
        time_series_df.iloc[0, :] = time_series_df.iloc[0, :].map(
            lambda value: _normalizer.norm_header(value)
        )

        time_series_df = time_series_df.rename(
            columns=time_series_df.iloc[0]
        ).drop(time_series_df.index[0])

        time_series_df.fillna(0, inplace=True)
        time_series_df.reset_index(inplace=True)
        time_series_df.rename(columns={'index': 'Numeric Index'}, inplace=True)

        time_series_df['Order'] = df_order

        time_series_df = pd.melt(
            time_series_df,
            id_vars=['Numeric Index', 'Index', 'Order'],
            var_name='Time',
            value_name='Value',
        )

        dfs.append(time_series_df)

        # Clean Time Series from raw dataframe
        df = clean_time_series_from_raw_df(df, metadata)
        df_order += 1

    if len(dfs) == 0:
        return pd.DataFrame()

    return pd.concat(dfs, ignore_index=True)


def find_all_time_series(
    raw_dataframes: Dict[Union[int, str], pd.DataFrame]
) -> pd.DataFrame:
    all_time_series = []

    for sheet_name, raw_df in raw_dataframes.items():
        df = find_time_series(raw_df)
        if not df.empty:
            df['Sheet Name'] = sheet_name
            all_time_series.append(df)

    return pd.concat(all_time_series, ignore_index=True)



import string
from typing import Dict, Union

import pandas as pd
from loguru import logger
from unidecode import unidecode

from cronus_eater import _converter, _extractor


def main():
    logger.info('Reading File ...')
    raw_dataframes = pd.read_excel(
        'tests/data/nubank_4Q22.xlsx', header=None, sheet_name=None
    )

    logger.info('Processing Time Series ...')
    times_series_df = _extractor.find_all_time_series(raw_dataframes)

    norm_index = (
        times_series_df['Index']
        .str.strip()
        .map(
            lambda value: unidecode(
                str(value).translate(str.maketrans('', '', string.punctuation))
            )
        )
        .str.replace(' ', '_')
    )

    norm_sheet_name = (
        times_series_df['Sheet Name']
        .str.strip()
        .map(
            lambda value: unidecode(
                str(value).translate(str.maketrans('', '', string.punctuation))
            )
        )
        .str.replace(' ', '_')
    )

    times_series_df['Chave'] = (
        norm_sheet_name
        + '_'
        + times_series_df['Numeric Index'].astype(str)
        + '_'
        + times_series_df['Order'].astype(str)
        + '_'
        + norm_index
    )

    times_series_df = times_series_df.pivot_table(
        index=['Chave'],
        columns='Time',
        values='Value',
    ).applymap(lambda value: _converter.blank_to_zero(value))

    times_series_df = times_series_df.reindex(
        sorted(times_series_df.columns.values, key=lambda value: value[1:]),
        axis=1,
    ).reset_index()

    logger.info('Saving Data ...')
    times_series_df.to_excel('xablau.xlsx', index=False)


if __name__ == '__main__':
    main()



from typing import Any

import pandas as pd

from cronus_eater import _validator


def blank_to_zero(value: Any) -> Any:
    if _validator.is_blank_value(value):
        return 0

    return value


def blank_to_na(value: Any) -> Any:
    if _validator.is_blank_value(value):
        return pd.NA

    return value



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cronus_eater']

package_data = \
{'': ['*']}

install_requires = \
['loguru>=0.6.0,<0.7.0', 'openpyxl>=3.0.10,<4.0.0', 'unidecode>=1.3.6,<2.0.0']

extras_require = \
{':python_full_version >= "3.7.1" and python_version < "3.8"': ['pandas==1.3.5'],
 ':python_version >= "3.8" and python_version < "3.12"': ['pandas>=1.5.0,<2.0.0']}

entry_points = \
{'console_scripts': ['cronus-eater = cronus_eater.__main__:main']}

setup_kwargs = {
    'name': 'cronus-eater',
    'version': '0.1.0a1',
    'description': 'A simple tool to get time series from spreadsheets',
    'long_description': '# Cronus Eater: A simple tool to get time series from spreadsheets\n\n<div align="center">\n  <img width="450" height="278" src="https://raw.githubusercontent.com/breno-jesus-fernandes/cronus-eater/5e3a88709eeb845fa24b1dff5b9d3039a88c3d9c/docs/img/cronus-eater-logo.png"><br>\n</div>\n\n[![Test](https://github.com/BrenoJesusFernandes/cronus-eater/actions/workflows/test.yaml/badge.svg)](https://github.com/BrenoJesusFernandes/cronus-eater/actions/workflows/test.yaml) [![Python: 3.7 | 3.8 | 3.9 | 3.10 | 3.11](https://img.shields.io/badge/Python-3.7%20%7C%203.8%20%7C%203.9%20%7C%203.10%20%7C%203.11-blue.svg)](https://pypi.org/project/cronus-eater/)  [![Code style: blue](https://img.shields.io/badge/code%20style-blue-blue.svg)](https://github.com/grantjenks/blue) [![Imports: isort](https://img.shields.io/badge/%20imports-isort-%231674b1?style=flat&labelColor=ef8336)](https://pycqa.github.io/isort/) [![LICENSE](https://img.shields.io/badge/license-MIT-green.svg)](https://github.com/BrenoJesusFernandes/cronus-eater/blob/main/LICENSE)\n\nExtract and normalize time series from any spreadsheet with differents patterns.\n\n\n### Where is the data I want?\n\n```python\n\nimport pandas as pd\n\nraw_dataframe = pd.read_excel(\'historical_series_3Q22.xlsx\')\nraw_dataframe.head()\n\n```\n\n|     |  0  |              1              |  2  |  3  |    4     |     5    |    6    |    7     |    8      |  9  |\n| --- | --- | --------------------------- | --- | --- | -------  | -------  | ------  | -------  | -------   | ----|\n| 0   | NaN | NaN                         | NaN | NaN |   NaN    |   NaN    |   NaN   |    NaN   |    NaN    | NaN |\n| 1   | NaN | Holdings Ltd.               | NaN | NaN |   NaN    |   NaN    |   NaN   |    NaN   |    NaN    | NaN |\n| 2   | NaN | NaN                         | NaN | NaN |   3Q22   |   2Q22   |   NaN   |   1Q22   |   2022    | NaN |\n| 3   | NaN | Amounts in thousands of R$  | NaN | NaN |   R$     |   R$     |   NaN   |    R$    |    R$     | NaN |\n| 4   | NaN | Cash Flow                   | NaN | NaN | $500.23  | $302.81  | $255.11 | $106.12  |  $900.00  | NaN |\n\n### Let\'s devours this times series  \n\n```python\n\nimport cronus_eater\ntimes_series_df = cronus_eater.find_time_series(raw_dataframe)\ntimes_series_df.head()\n\n```\n\n\n|     | Numeric Index | Label Index | Table Order | Time | Value   |\n| --- | ------------- | ---------   | ----------- | ---- | ------- |\n| 0   |  4            | Cash Flow   | 1           | 4Q22 | 302.81  |\n| 1   |  4            | Cash Flow   | 1           | 2Q22 | 255.11  |\n| 2   |  4            | Cash Flow   | 1           | 1Q22 | 106.12  |\n| 3   |  4            | Cash Flow   | 1           | 2017 | 900.00  |\n\n\n## Where to get it\n\nThe source code is currently hosted on GitHub at: <https://github.com/breno-jesus-fernandes/cronus-eater>\n\nBinary installers for the latest released version is going to available at the [Python Package Index (PyPI)](https://pypi.org/project/cronus-eater).\n\n\n```sh\npip install cronus-eater\n# or through poetry\npoetry add cronus-eater\n```\n\n## License\n\n[MIT](LICENSE)\n\n## Documentation\n\nYou can find [here](docs/)\n\n\n ',
    'author': 'Breno Fernandes',
    'author_email': 'breno.de.jesus.fernandes@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/breno-jesus-fernandes/cronus-eater',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7.1,<3.12',
}


setup(**setup_kwargs)



from datetime import datetime
from typing import Any, List

import numpy as np
import pandas as pd

from cronus_eater import _validator
from cronus_eater.model import TimeSeries


def norm_blank_value(value: Any) -> Any:
    if _validator.is_blank_value(value):
        return pd.NA

    return value


def norm_header(value: Any) -> Any:
    if isinstance(value, datetime):
        value = f'{pd.Timestamp(value).quarter}T{str(value.year)[2:]}'
        return value

    return value
